import base58
import hashlib


def parse_cf_v1_address(address: str) -> tuple[int, int, int, bytes, bytes, bytes]:
    bdata: bytes = base58.b58decode(address)
    version = bdata[0]
    net_id = int.from_bytes((bdata[1:9]), byteorder='little')
    sign_id = int.from_bytes((bdata[9:13]), byteorder='little')
    public_hash = bdata[13:45]
    control_hash = bdata[45:]
    hash = hashlib.sha3_256()
    hash.update(bdata[:45])
    summary_hash = hash.digest()
    if summary_hash != control_hash:
        raise ValueError(f"Address={address} not valid")
    return version, net_id, sign_id, public_hash, summary_hash, control_hash