from __future__ import annotations
from typing import TypeVar, TYPE_CHECKING
from DAP.Crypto import Sign, Pkey, Cert
from CellFrame.Chain import ChainAddr

if TYPE_CHECKING:
    from .net import CFNet

# from DAP import Crypto
from pycfhelpers.logger import log

# log.message(f"{dir(Crypto)=}")


class CFSign:
    # __slots__ = ["_origin_sign", "net", "type", "pkey", "size", "pkey_hash", "address"]

    def __init__(self, sign: Sign, net: CFNet):
        self._origin_sign = sign
        self.net = net
        self.type = str(sign.type)
        self.pkey = CFPkey(sign.pkey)
        self.size = sign.size
        self.pkey_hash = str(self._origin_sign.pkeyHash)
        self.address = str(ChainAddr.fill(self._origin_sign.type, self._origin_sign.pkeyHash, self.net._origin_net.id))

    def serialize(self) -> bytes:
        return self._origin_sign.toBytes()

    @staticmethod
    def deserialize(data: bytes, net: CFNet) -> CFSign:
        return CFSign(Sign.fromBytes(data), net)


class CFPkey:
    def __init__(self, pkey: Pkey):
        self._origin_pkey = pkey
        self.type = pkey.type
        self.size = pkey.size
        self.hash = str(pkey.hash)

    def _to_bytes(self) -> bytes:
        return self._origin_pkey.toBytes()

    @staticmethod
    def _from_bytes(data: bytes, /) -> CFPkey:
        return CFPkey(Pkey.fromBytes(data))


class CFKey:
    def __init__(self, key: Key):
        self._origin_key = key


class CFCertificate:
    def __init__(self, certificate_name: str):
        certificate = Cert.load(certificate_name)
        if certificate is None:
            raise ValueError(f"Certificate with name={certificate_name} not found")
        self._origin_certificate = certificate
        self.pkey = CFPkey(certificate.pkey())  # public
        self.key = certificate.key  # private

    @property
    def private_key(self) -> CFKey | None:
        """
        Alias for key
        """
        return self.key

    @property
    def public_key(self) -> CFPkey:
        """
        Alias for pkey
        """
        return self.pkey

    def delete(self) -> None:
        self._origin_certificate.delete()
